from services._version import ___version___ as version
from pyat.schema import create_engine
from pyramid.config import Configurator
from pyramid_beaker import session_factory_from_settings
from sqlalchemy.orm import scoped_session, sessionmaker
from zope.sqlalchemy import ZopeTransactionExtension
from pyramid.renderers import JSONP

DB = {}


def base_repr(s):
    r""" Return the string representation of this class.

        :return: The string representation of self.
        """
    repr_str_list = ['{c}(']
    temp = ['{0}={{s.{0}}}'.format(key) for key in vars(s)]
    self_list = ', '.join(temp)
    repr_str_list.append(self_list)
    repr_str_list.append(')')

    return ''.join(repr_str_list).format(c=s.__class__.__name__, s=s)


def main(global_config, **settings):
    DB['SDM'] = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
    DB['NGAS'] = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
    DB['NGAS_NAASC'] = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
    DB['VLASS'] = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))

    DB['SDM'].configure(bind=create_engine('SDM'))
    DB['NGAS'].configure(bind=create_engine('NGAS'))
    DB['NGAS_NAASC'].configure(bind=create_engine('NGAS_NAASC'))
    DB['VLASS'].configure(bind=create_engine('VLASS'))

    with Configurator(settings=settings) as config:
        session_factory = session_factory_from_settings(settings)
        config.set_session_factory(session_factory)

        config.add_renderer('jsonp', JSONP(param_name='callback'))

        config.include('pyramid_beaker')

        config.add_route('product_location_service', '/location')
        config.add_route('product_authorization_service', '/authorized')

        config.add_route('get_solr_facets', '/restapi_get_execution_block_facets')
        config.add_route('get_eb_project_view', '/restapi_get_eb_project_view')
        config.add_route('get_paged_exec_blocks', '/restapi_get_paged_exec_blocks')
        config.add_route('get_paged_images', '/restapi_get_paged_images')
        config.add_route('get_full_exec_block_details', '/restapi_get_full_exec_block_details')
        config.add_route('get_paged_alma_mouses', '/restapi_get_paged_alma_mouses')
        config.add_route('echo_json_params', '/echo_json_params')
        config.add_route('get_splash_message', '/get_splash_message')
        config.add_route('acknowledge_splash_message', '/acknowledge_splash_message')
        config.add_route('get_casa_version_list', '/restapi_get_casa_version_list')
        config.add_route('get_release_notes_url', '/restapi_get_release_notes_url')
        config.add_route('get_application_parameters', '/restapi_get_application_parameters')
        config.add_route('get_pre_release_feature_flag', '/restapi_get_pre_release_feature_flag')
        config.add_route('get_show_all_thumbnails_flag', '/restapi_get_show_all_thumbnails_flag')
        config.add_route('get_rh_url', '/restapi_get_rh_url')
        config.add_route('resolver_request', '/restapi_resolver_request')

        config.add_route('mous_details_view', '/restapi_mous_details_view')
        config.add_route('data_quality_notes_view', '/restapi_data_quality_notes')

        config.add_route('image_metadata_service', '/image_metadata')
        config.add_route('image_precursor_service', '/image_precursor')

        config.scan('services.locator')
        config.scan('services.auth')
        config.scan('services.solr')
        config.scan('services.client_services')
        config.scan('services.mous_details')
        config.scan('services.ingestion_metadata')
        return config.make_wsgi_app()

